#include <stdio.h>
#include <getopt.h>
#include <unistd.h>

#include "shaperd.h"

static struct option options[] =
{
    {"config",required_argument,0,'c'},
    {"start_speed",optional_argument,0,'H'},
    {"bandwidth_checksum",optional_argument,0,'C'},
    {"squid_support",optional_argument,0,'s'},
    {"divide_upload",optional_argument,0,'D'},
    {"extended_queue",optional_argument,0,'e'},
    {"nomasq",optional_argument,0,'m'},
    {"class_type",required_argument,0,'t'},
    {"upload_class_type",required_argument,0,'T'},
    {"even_division",optional_argument,0,'E'},
    {"continous_control",optional_argument,0,'f'},
    {"jump_type",required_argument,0,'j'},
    {"speed_unit",required_argument,0,'u'},
    {"delay",required_argument,0,'r'},
    {"write_delay",required_argument,0,'w'},
    {"debug",required_argument,0,'d'},
    {"version",no_argument,0,'v'},
    {"help",no_argument,0,'h'},
    {0,0,0,0}
};

static char *help_msg=
"Usage: shaperd [options] [commands]\n"
"Options:\n"
"  -c, --config=0-9                 number of shaperd config (deflt. 0)\n"
"  -H, --start_speed=n              speed start of new user (deflt. 0)\n"
"  -C, --bandwidth_checksum[=yes|no]\n"
"  -s, --squid_support[=yes|no|rules_only]     PROXY support switch (deflt. no)\n"
"  -D, --divide_upload[=yes|no]     divide upload bandwidth (deflt. no)\n"
"  -e, --extended_queue[=yes|no]    extended dividing which create 4 class\n"
"                                   for every user with high/med/low/mass\n"
"                                   priorities (default: no)\n"
"  -m, --nomasq[=yes|no]            disable masquerade connections \n"
"                                   tracking (default yes)\n"
"  -t, --class_type=cbq|htb         download class type (deflt. cbq)\n"
"  -T, --upload_class_type=cbq|htb  upload class type (deflt. cbq)\n"
"  -E, --even_division[=yes|no]     even division of bandwidth (deflt. no)\n"
"  -f, --continuous_control[=yes|no]continuous QoS control (deflt. yes)\n"
"  -j, --jump_type=n                multiplier of speed jump (deflt. 0)\n"
"  -u, --speed_unit=bit|kbit|mbit   speed extension (deflt. bit)\n"
"  -r, --delay=seconds              cycle delay time (deflt 10)\n"
"  -w, --write_delay=n              after how many cycles to write (0 disable)\n"
"                                   bitrate_user_sh.x.old file (deflt. 6)\n"
"  -d, --debug=0|1|2|3|4            debug level (deflt. 0)\n"
"  -v, --version                    print package version\n"
"  -h, --help                       this help message\n"
"\n"
"When you ommit the optional values \"yes\" or \"no\", the shaperd will\n"
"set them to \"yes\"\n"
"\n"
"Commands:\n"
"  shownat      show current masqueraded connections (without ignore)\n"
"  shownat_all  show all current masqueraded connections\n"
"\n";

void parse_cmdline (int argc, char *argv[], struct shaperd_options *opts)
{
    int o;
    int i=0;
    
    while (1) {

	o=getopt_long(argc,argv,
		      "c:H::C::s::D::e::m::t:T:E::f::j:u:r:w:d:vh",
		      options,&i);

	if (o==-1)
	    break;

	switch(o) {

	case 'c':
	    opts->config=shaperd_strtoi(optarg);
	    break;

	case 'H':
	    if(optarg)
		opts->start_speed=shaperd_strtoi(optarg);
	    break;

	case 'C':
	    if(optarg)
		opts->check_sum_of_bandwidth=shaperd_strtobool(optarg);
	    break;

	case 's':
	    if(optarg)
//		opts->squid_support=shaperd_strtobool(optarg);
		opts->squid_support=shaperd_strtoi(optarg);
	    break;

	case 'D':
	    if(optarg)
		opts->divide_upload=shaperd_strtobool(optarg);
	    break;

	case 'e':
	    if(optarg)
		opts->extended_queue=shaperd_strtobool(optarg);
	    break;

	case 'm':
	    if(optarg)
		opts->nomasq=shaperd_strtobool(optarg);
	    break;

	case 't':
	    opts->class_type=shaperd_strtoclass(optarg);
	    break;

	case 'T':
	    opts->upload_class_type=shaperd_strtoclass(optarg);
	    break;

	case 'E':
	    opts->even_division=shaperd_strtobool(optarg);
	    break;

	case 'f':
	    opts->continous_control=shaperd_strtobool(optarg);
	    break;

	case 'j':
	    opts->jump_type=shaperd_strtoi(optarg);
	    break;

	case 'u':
	    opts->speed_ext=shaperd_strtospeedu(optarg);
	    if(opts->speed_ext)
		break;
	    fprintf(stderr,"Wrong speed unit value.\n");
	    _exit(1);

	case 'r':
	    opts->delay=shaperd_strtoi(optarg);
	    break;

	case 'w':
	    opts->olddelay=shaperd_strtoi(optarg);
	    break;

	case 'd':
	    opts->debug=shaperd_strtoi(optarg);
	    if ((opts->debug >= 0)&&(opts->debug <= 4))
		break;
	    fprintf(stderr,"Wrong debug level value.\n");
	    _exit(1);
	    
	case 'v':
	    printf ("shaperd version %s\n",SHAPERD_VERSION);
	    _exit(0);

	case 'h':
	    puts(help_msg);
	    fflush(stdout);
	    _exit(0);

	case '?':
	    puts(help_msg);
	    fflush(stdout);
	    _exit(1);
	}
    }
}
